using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

namespace CellStyles
{
    public partial class StyleDemo : Form
    {
        static private StringFormat _format = new StringFormat(StringFormatFlags.MeasureTrailingSpaces);
        static private Font _classFont = new Font("Arial", 10, FontStyle.Bold);
        static private Font _propFont = new Font("Arial", 10, FontStyle.Italic);
        static private Font _textFont = new Font("Arial", 10, FontStyle.Regular);
        static private DataGridViewCellStyle[] defaultStyles = new DataGridViewCellStyle[10];

        public StyleDemo()
        {
            InitializeComponent();
            _format.Alignment = StringAlignment.Near;
            _format.LineAlignment = StringAlignment.Center;
        }

        private void StyleDemo_Load(object sender, EventArgs e)
        {
            this._dgv.Rows.Add(4);
            defaultStyles[0] = this._dgv.DefaultCellStyle.Clone();
            defaultStyles[1] = this._dgv.ColumnHeadersDefaultCellStyle.Clone();
            defaultStyles[2] = this._dgv.RowHeadersDefaultCellStyle.Clone();
            defaultStyles[3] = this._dgv.Columns[1].HeaderCell.Style.Clone();
            defaultStyles[4] = this._dgv.Rows[1].HeaderCell.Style.Clone();
            defaultStyles[5] = this._dgv.Rows[1].Cells[1].Style.Clone();
            defaultStyles[6] = this._dgv.Columns[1].DefaultCellStyle.Clone();
            defaultStyles[7] = this._dgv.RowsDefaultCellStyle.Clone();
            defaultStyles[8] = this._dgv.AlternatingRowsDefaultCellStyle.Clone();
            defaultStyles[9] = this._dgv.Rows[1].DefaultCellStyle.Clone();
            this.ShowColorInfo();
        }

        private void button_All_Paint(object sender, PaintEventArgs e)
        {
            string tag, className, propName, comment;
            Rectangle start = e.ClipRectangle;

            Button bt = sender as Button;
            if(bt==null)
                return;
            tag = bt.Tag.ToString();
            if(string.IsNullOrEmpty(tag))
                return;
            string[] arr = tag.Split('.');
            if(arr.Length != 2 && arr.Length != 3)
                return;
            className = arr[0]; propName = arr[1];
            DarwNextString(e, ref start, "Set ", _textFont, false);
            DarwNextString(e, ref start, className + ".", _classFont, true);
            DarwNextString(e, ref start, propName, _propFont, true);
            switch(tag)
            {
                case "DataGridView.DefaultCellStyle":
                    comment = " - for all ordinary cells.";
                    break;
                case "DataGridView.ColumnHeadersDefaultCellStyle":
                    comment = " - for all column's header cells.";
                    break;
                case "DataGridView.RowHeadersDefaultCellStyle":
                    comment = " - for all row's header cells.";
                    break;
                case "DataGridViewCell.Style.CH":
                    comment = " - for second column's header cell.";
                    break;
                case "DataGridViewCell.Style.RH":
                    comment = " - for second row's header cell.";
                    break;
                case "DataGridViewCell.Style":
                    comment = " - for ordinary cell in second column & second row.";
                    break;
                case "DataGridViewColumn.DefaultCellStyle":
                    comment = " - for all ordinary cells in second column";
                    break;
                case "DataGridView.RowsDefaultCellStyle":
                    comment = " - for all ordinary cells in all rows";
                    break;
                case "DataGridView.AlternatingRowsDefaultCellStyle":
                    comment = " - for all ordinary cells in odd (1,3,5,...) rows";
                    break;
                case "DataGridViewRow.DefaultCellStyle":
                    comment = " - for all ordinary cells in second row";
                    break;
                default:
                    comment=string.Empty;
                    break;
            }
            DarwNextString(e, ref start, comment, _textFont, false);
        }

        private void DarwNextString(PaintEventArgs e, ref Rectangle start, string text, Font fnt, bool precise)
        {
            int offset;
            e.Graphics.DrawString(text, fnt, Brushes.Black, start, _format);
            offset = (int)e.Graphics.MeasureString(text, fnt, new SizeF(start.Width, start.Height), precise ? StringFormat.GenericTypographic : _format).Width;
            start = new Rectangle(start.X + offset, start.Y, start.Width - offset, start.Height);
        }

        private void button_All_Click(object sender, EventArgs e)
        {
            string tag;

            Button bt = sender as Button;
            if(bt == null)
                return;
            Color targetColor = bt.BackColor;
            tag = bt.Tag.ToString();
            if(string.IsNullOrEmpty(tag) || targetColor==Color.Empty)
                return;
            switch(tag)
            {
                case "DataGridView.DefaultCellStyle":
                    this._dgv.DefaultCellStyle.BackColor = this._dgv.DefaultCellStyle.SelectionBackColor = targetColor;
                    break;
                case "DataGridView.ColumnHeadersDefaultCellStyle":
                    this._dgv.ColumnHeadersDefaultCellStyle.BackColor = this._dgv.ColumnHeadersDefaultCellStyle.SelectionBackColor = targetColor;
                    break;
                case "DataGridView.RowHeadersDefaultCellStyle":
                    this._dgv.RowHeadersDefaultCellStyle.BackColor = this._dgv.RowHeadersDefaultCellStyle.SelectionBackColor = targetColor;
                    break;
                case "DataGridViewCell.Style.CH":
                    this._dgv.Columns[1].HeaderCell.Style.BackColor = this._dgv.Columns[1].HeaderCell.Style.SelectionBackColor = targetColor;
                    break;
                case "DataGridViewCell.Style.RH":
                    this._dgv.Rows[1].HeaderCell.Style.BackColor = this._dgv.Rows[1].HeaderCell.Style.SelectionBackColor = targetColor;
                    break;
                case "DataGridViewCell.Style":
                    this._dgv.Rows[1].Cells[1].Style.BackColor = this._dgv.Rows[1].Cells[1].Style.SelectionBackColor = targetColor;
                   break;
               case "DataGridViewColumn.DefaultCellStyle":
                   this._dgv.Columns[1].DefaultCellStyle.BackColor = this._dgv.Columns[1].DefaultCellStyle.SelectionBackColor = targetColor;
                   break;
                case "DataGridView.RowsDefaultCellStyle":
                    this._dgv.RowsDefaultCellStyle.BackColor = this._dgv.RowsDefaultCellStyle.SelectionBackColor = targetColor;
                    break;
                case "DataGridView.AlternatingRowsDefaultCellStyle":
                    this._dgv.AlternatingRowsDefaultCellStyle.BackColor = this._dgv.AlternatingRowsDefaultCellStyle.SelectionBackColor = targetColor;
                    break;
                case "DataGridViewRow.DefaultCellStyle":
                    this._dgv.Rows[1].DefaultCellStyle.BackColor = this._dgv.Rows[1].DefaultCellStyle.SelectionBackColor = targetColor;
                    break;
            }
            this.ShowColorInfo();
        }

        private void _btDrop_Click(object sender, EventArgs e)
        {
            this._dgv.DefaultCellStyle = defaultStyles[0].Clone();
            this._dgv.ColumnHeadersDefaultCellStyle = defaultStyles[1].Clone();
            this._dgv.RowHeadersDefaultCellStyle = defaultStyles[2].Clone();
            this._dgv.Columns[1].HeaderCell.Style = defaultStyles[3].Clone();
            this._dgv.Rows[1].HeaderCell.Style = defaultStyles[4].Clone();
            this._dgv.Rows[1].Cells[1].Style = defaultStyles[5].Clone();
            this._dgv.Columns[1].DefaultCellStyle = defaultStyles[6].Clone();
            this._dgv.RowsDefaultCellStyle = defaultStyles[7].Clone();
            this._dgv.AlternatingRowsDefaultCellStyle = defaultStyles[8].Clone();
            this._dgv.Rows[1].DefaultCellStyle = defaultStyles[9].Clone();
            this.ShowColorInfo();
        }

        private void ShowColorInfo()
        {
            Color inhColor = this._dgv.Rows[1].Cells[1].InheritedStyle.BackColor;
            Color styColor = this._dgv.Rows[1].Cells[1].Style.BackColor;
            if(inhColor == Color.Empty)
            {
                this._lbInheritedStyle.ForeColor = Color.Black;
                this._lbInheritedStyle.BackColor = this.BackColor;
            }
            else
                this._lbInheritedStyle.ForeColor = this._lbInheritedStyle.BackColor = inhColor;
            if(styColor == Color.Empty)
            {
                this._lbStyle.ForeColor = Color.Black;
                this._lbStyle.BackColor = this.BackColor;
            }
            else
                this._lbStyle.ForeColor = this._lbStyle.BackColor = styColor;
        }
    }
}